/**
 * When followed by !, that means the checker will attempt to cast the value to
 * the specified type in-place before checking the type. For example, if
 * {
 *   count: '123'
 * }
 * is tested against:
 * {
 *   count: 'number!'
 * }
 * The value will be converted to number and then check if it is indeed a number.
 * The object will be modified to
 * {
 *   count: 123
 * }
 * Special Rules:
 * 1. number!: NaN and Infinity will be converted to undefined
 * 2. boolean!: anything other than 'true' or 'false' will be converted to undefined.
 * 3. string!: empty string will be converted to undefined.
 */
export declare type PrimitiveTypes = 'string!' | 'string' | 'number' | 'number!' | 'boolean' | 'boolean!' | 'object';
/**
 * The key must be a valid variable name with an optional trailing question
 * mark. The question mark indicates that the arg is optional but if exists
 * it's value will be tested against the specified type.
 */
export interface Interface {
    [key: string]: PrimitiveTypes | Interface;
}
/**
 * Get the list of paths of mismatched properties.
 * @param values       The object to test.
 * @param fieldSpecs   The interface.
 */
export declare function getMismatchedFields(values: Object, fieldSpecs: Interface): Array<string>;
/**
 * Check if the object complies with the interface.
 * @param values       The object to test.
 * @param fieldSpecs   The interface spec.
 */
export declare function validate<T>(values: Object, fieldSpecs: Interface): values is T;
