"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
function isOptionalParam(name) {
    return name.charAt(name.length - 1) === '?';
}
/**
 * Get the list of paths of mismatched properties.
 * @param values       The object to test.
 * @param fieldSpecs   The interface.
 */
function getMismatchedFields(values, fieldSpecs) {
    var nestedSpecNames = Object.keys(fieldSpecs).filter(function (name) {
        return _.isObject(fieldSpecs[name]);
    });
    var nestedMismatched = [].concat.apply([], nestedSpecNames.map(function (specName) {
        var isOptional = isOptionalParam(specName);
        var valueKey = specName.replace(/\?/g, '');
        if (!values[valueKey]) {
            return (isOptional) ? [] : [valueKey];
        }
        return getMismatchedFields(values[valueKey], fieldSpecs[specName]).map(function (name) { return valueKey + "." + name; }); // append the specName to indicate scope.
    }));
    var mismatched = Object.keys(fieldSpecs).map(function (specName) {
        var isOptional = isOptionalParam(specName);
        var specType = fieldSpecs[specName];
        var valueKey = specName.replace(/\?/g, '');
        var value = values[valueKey];
        if (isOptional && !value || _.isObject(fieldSpecs[specName])) {
            return null;
        }
        var convertedValue = value;
        if (_.isString(value)) {
            switch (specType) {
                case 'number!':
                    specType = 'number';
                    convertedValue = _.toNumber(value);
                    !_.isFinite(convertedValue) && (convertedValue = undefined);
                    break;
                case 'boolean!':
                    specType = 'boolean';
                    value === 'true' && (convertedValue = true);
                    value === 'false' && (convertedValue = false);
                    !_.isBoolean(convertedValue) && (convertedValue = undefined);
                    break;
                case 'string!':
                    specType = 'string';
                    convertedValue = (convertedValue === '') ? undefined : convertedValue;
            }
        }
        var typeMatch = typeof convertedValue === specType;
        values[valueKey] = convertedValue;
        if (typeMatch) {
            return null;
        }
        return valueKey;
    }).filter(Boolean);
    return mismatched.concat(nestedMismatched);
}
exports.getMismatchedFields = getMismatchedFields;
/**
 * Check if the object complies with the interface.
 * @param values       The object to test.
 * @param fieldSpecs   The interface spec.
 */
function validate(values, fieldSpecs) {
    return getMismatchedFields(values, fieldSpecs).length === 0;
}
exports.validate = validate;
